'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _common = require('../prelude/common.js');

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _log = require('./log.js');

var _package = require('../package.json');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var bootstrapText = _fsExtra2.default.readFileSync(require.resolve('../prelude/bootstrap.js'), 'utf8').replace('%VERSION%', _package.version); /* eslint-disable complexity */

var commonText = _fsExtra2.default.readFileSync(require.resolve('../prelude/common.js'), 'utf8');

function itemsToText(items) {
  var len = items.length;
  return len.toString() + (len % 10 === 1 ? ' item' : ' items');
}

function hasAnyStore(record) {
  // discarded records like native addons
  var _arr = [_common.STORE_BLOB, _common.STORE_CONTENT, _common.STORE_LINKS, _common.STORE_STAT];
  for (var _i = 0; _i < _arr.length; _i++) {
    var store = _arr[_i];
    if (record[store]) return true;
  }
  return false;
}

exports.default = function () {
  var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(_ref2) {
    var records = _ref2.records,
        entrypoint = _ref2.entrypoint;

    var stripe, file, record, _arr2, _i2, store, _value, buffer, newStat, _buffer, disclosed, value, prelude;

    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            stripe = [];
            _context.t0 = _regenerator2.default.keys(records);

          case 2:
            if ((_context.t1 = _context.t0()).done) {
              _context.next = 23;
              break;
            }

            file = _context.t1.value;
            record = records[file];

            if (hasAnyStore(record)) {
              _context.next = 7;
              break;
            }

            return _context.abrupt('continue', 2);

          case 7:
            (0, _assert2.default)(record[_common.STORE_STAT], 'packer: no STORE_STAT');
            (0, _assert2.default)(record[_common.STORE_BLOB] || record[_common.STORE_CONTENT] || record[_common.STORE_LINKS]);

            _arr2 = [_common.STORE_BLOB, _common.STORE_CONTENT, _common.STORE_LINKS, _common.STORE_STAT];
            _i2 = 0;

          case 11:
            if (!(_i2 < _arr2.length)) {
              _context.next = 20;
              break;
            }

            store = _arr2[_i2];
            _value = record[store];

            if (_value) {
              _context.next = 16;
              break;
            }

            return _context.abrupt('continue', 17);

          case 16:

            if (store === _common.STORE_BLOB || store === _common.STORE_CONTENT) {
              if (record.body === undefined) {
                stripe.push({ file: file, store: store });
              } else if (Buffer.isBuffer(record.body)) {
                stripe.push({ file: file, store: store, buffer: record.body });
              } else if (typeof record.body === 'string') {
                stripe.push({ file: file, store: store, buffer: Buffer.from(record.body) });
              } else {
                (0, _assert2.default)(false, 'packer: bad STORE_BLOB/STORE_CONTENT');
              }
            } else if (store === _common.STORE_LINKS) {
              if (Array.isArray(_value)) {
                buffer = Buffer.from((0, _stringify2.default)(_value));

                stripe.push({ file: file, store: store, buffer: buffer });
              } else {
                (0, _assert2.default)(false, 'packer: bad STORE_LINKS');
              }
            } else if (store === _common.STORE_STAT) {
              if ((typeof _value === 'undefined' ? 'undefined' : (0, _typeof3.default)(_value)) === 'object') {
                newStat = (0, _assign2.default)({}, _value);

                newStat.atime = _value.atime.getTime();
                newStat.mtime = _value.mtime.getTime();
                newStat.ctime = _value.ctime.getTime();
                newStat.birthtime = _value.birthtime.getTime();
                newStat.isFileValue = _value.isFile();
                newStat.isDirectoryValue = _value.isDirectory();
                _buffer = Buffer.from((0, _stringify2.default)(newStat));

                stripe.push({ file: file, store: store, buffer: _buffer });
              } else {
                (0, _assert2.default)(false, 'packer: bad STORE_STAT');
              }
            } else {
              (0, _assert2.default)(false, 'packer: unknown store');
            }

          case 17:
            _i2++;
            _context.next = 11;
            break;

          case 20:

            if (record[_common.STORE_CONTENT]) {
              disclosed = (0, _common.isDotJS)(file) || (0, _common.isDotJSON)(file);

              _log.log.debug(disclosed ? 'The file was included as DISCLOSED code (with sources)' : 'The file was included as asset content', file);
            } else if (record[_common.STORE_BLOB]) {
              _log.log.debug('The file was included as bytecode (no sources)', file);
            } else if (record[_common.STORE_LINKS]) {
              value = record[_common.STORE_LINKS];

              _log.log.debug('The directory files list was included (' + itemsToText(value) + ')', file);
            }
            _context.next = 2;
            break;

          case 23:
            prelude = 'return (function (REQUIRE_COMMON, VIRTUAL_FILESYSTEM, DEFAULT_ENTRYPOINT) { ' + bootstrapText + '\n})(function (exports) {\n' + commonText + '\n},\n' + '%VIRTUAL_FILESYSTEM%' + '\n,\n' + '%DEFAULT_ENTRYPOINT%' + '\n);';
            return _context.abrupt('return', { prelude: prelude, entrypoint: entrypoint, stripe: stripe });

          case 25:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function (_x) {
    return _ref.apply(this, arguments);
  };
}();